@@ -67,7 +67,7 @@ my $diff_ignore_default_regex = '
 # Ignore baz-style junk files or directories
 (?:^|/),,.*(?:$|/.*$)|
 # File-names that should be ignored (never directories)
-(?:^|/)(?:DEADJOE|\.arch-inventory|\.(?:bzr|cvs|hg|git)ignore)$|
+(?:^|/)(?:DEADJOE|\.arch-inventory|\.(?:bzr|cvs|hg|git|mtn-)ignore)$|
 # File or directory names that should be ignored
 (?:^|/)(?:CVS|RCS|\.deps|\{arch\}|\.arch-ids|\.svn|
 \.hg(?:tags|sigs)?|_darcs|\.git(?:attributes|modules|review)?|
@@ -112,6 +112,7 @@ our @tar_ignore_default_pattern = qw(
 .hgsigs
 .hgtags
 .mailmap
+.mtn-ignore
 .shelf
 .svn
 CVS
@@ -280,14 +281,16 @@ sub upgrade_object_type {
     my $format = $self->{fields}{'Format'};
 
     if ($format =~ /^([\d\.]+)(?:\s+\((.*)\))?$/) {
-        my ($version, $variant, $major, $minor) = ($1, $2, $1, undef);
+        my ($version, $variant) = ($1, $2);
 
         if (defined $variant and $variant ne lc $variant) {
             error(g_("source package format '%s' is not supported: %s"),
                   $format, g_('format variant must be in lowercase'));
         }
 
-        $major =~ s/\.[\d\.]+$//;
+        my $major = $version =~ s/\.[\d\.]+$//r;
+        my $minor;
+
         my $module = "Dpkg::Source::Package::V$major";
         $module .= '::' . ucfirst $variant if defined $variant;
         eval qq{
